﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using RedBlackTrees;

namespace TestBstBalance {

  public class Tester {
    internal int shortest;
    internal int longest;
    internal int sumOflengths;
    internal int currentLength;
    internal int externalNodeCount;

    public void CalculateStats(IBinaryNode node) {
      currentLength++;
      if (node == null) {
        externalNodeCount++;
        if (currentLength < shortest)
          shortest = currentLength;
        if (currentLength > longest)
          longest = currentLength;
        sumOflengths += currentLength;
      }
      else {
        CalculateStats(node.Left);
        CalculateStats(node.Right);
      }
      currentLength--;
    }

    public void Run() {
      Console.WriteLine("Testing binary search tree balancing");

      Random rg = new Random();

      using (StreamWriter sw = new StreamWriter("BalanceResults.csv")) {

        for (int loop = 0; loop < 100; loop++) {
          Console.WriteLine("Creating BSTree");
          BinarySearchTree<int, int> tree = new BinarySearchTree<int, int>();

          Console.WriteLine("Adding 2047 random values...");
          int key;
          for (int i = 0; i < 2048- 1; i++) {
            do {
              key = rg.Next(300000);

            } while (tree.Contains(key));
            tree.Insert(key, key + 1);
          }

          Console.WriteLine("Calculating stats...");
          shortest = int.MaxValue;
          longest = -1;
          sumOflengths = 0;
          externalNodeCount = 0;
          currentLength = -1; // so that the root has path length 0
          CalculateStats(tree.Root);

          sw.WriteLine(String.Format("{0},{1},{2}", shortest, longest, (float)sumOflengths / externalNodeCount));
          
        }

      }

      Console.WriteLine("...done.");
    }
  }

  
  class Program {
    static void Main(string[] args) {
      Tester tester = new Tester();
      tester.Run();

      Console.ReadLine();
    }
  }
}
